@@ -17,7 +17,7 @@ from supervised.utils.config import LOG_LEVEL
 logger = logging.getLogger(__name__)
 logger.setLevel(LOG_LEVEL)
 
-from catboost import CatBoostClassifier, CatBoostRegressor
+from catboost import CatBoostClassifier, CatBoostRegressor, CatBoost
 import catboost
 
 
@@ -31,22 +31,25 @@ class CatBoostAlgorithm(BaseAlgorithm):
         self.library_version = catboost.__version__
         self.snapshot_file_path = "training_snapshot"
 
-        self.rounds = additional.get("trees_in_step", 10)
-        self.max_iters = additional.get("max_steps", 500)
-
+        self.explain_level = params.get("explain_level", 0)
+        self.rounds = additional.get("max_rounds", 10000)
+        self.max_iters = 1
+        self.early_stopping_rounds = additional.get("early_stopping_rounds", 50)
+        
         Algo = CatBoostClassifier
         loss_function = "Logloss"
         if self.params["ml_task"] == MULTICLASS_CLASSIFICATION:
             loss_function = "MultiClass"
         elif self.params["ml_task"] == REGRESSION:
-            loss_function = self.params.get("loss_function", "RMSE")
+            loss_function = "RMSE"  # self.params.get("loss_function", "RMSE")
             Algo = CatBoostRegressor
 
         self.learner_params = {
             "learning_rate": self.params.get("learning_rate", 0.1),
-            "depth": self.params.get("depth", 6),
-            "rsm": self.params.get("rsm", 1),
-            "l2_leaf_reg": self.params.get("l2_leaf_reg", 3),
+            "depth": self.params.get("depth", 3),
+            "min_data_in_leaf": self.params.get("min_data_in_leaf", 1),
+            "rsm": self.params.get("rsm", 1.0),
+            "subsample": self.params.get("subsample", 1.0),
             "random_seed": self.params.get("seed", 1),
             "loss_function": loss_function,
         }
@@ -56,7 +59,7 @@ class CatBoostAlgorithm(BaseAlgorithm):
             learning_rate=self.learner_params["learning_rate"],
             depth=self.learner_params["depth"],
             rsm=self.learner_params["rsm"],
-            l2_leaf_reg=self.learner_params["l2_leaf_reg"],
+            min_data_in_leaf=self.learner_params["min_data_in_leaf"],
             loss_function=self.learner_params["loss_function"],
             verbose=False,
             allow_writing_files=False,
@@ -65,19 +68,38 @@ class CatBoostAlgorithm(BaseAlgorithm):
 
         logger.debug("CatBoostAlgorithm.__init__")
 
-    def fit(self, X, y):
+    def fit(self, X, y, X_validation=None, y_validation=None, log_to_file=None):
         if self.cat_features is None:
             self.cat_features = []
             for i in range(X.shape[1]):
                 if PreprocessingUtils.is_categorical(X.iloc[:, i]):
                     self.cat_features += [i]
 
+        eval_set = None 
+        if X_validation is not None and y_validation is not None:
+            eval_set = (X_validation, y_validation)
+
         self.model.fit(
             X,
             y,
             cat_features=self.cat_features,
             init_model=None if self.model.tree_count_ is None else self.model,
+            eval_set=eval_set,
+            early_stopping_rounds=self.early_stopping_rounds,
+            verbose_eval=False,
         )
+        if log_to_file is not None:
+            
+            metric_name = list(self.model.evals_result_["learn"].keys())[0]
+            result = pd.DataFrame(
+                {
+                    "iteration": range(len(self.model.evals_result_["learn"][metric_name])),
+                    "train": self.model.evals_result_["learn"][metric_name],
+                    "validation": self.model.evals_result_["validation"][metric_name],
+                }
+            )
+            result.to_csv(log_to_file, index=False, header=False)
+
 
     def predict(self, X):
         if self.params["ml_task"] == BINARY_CLASSIFICATION:
@@ -95,16 +117,13 @@ class CatBoostAlgorithm(BaseAlgorithm):
 
     def load(self, model_file_path):
         logger.debug("CatBoostLearner load model from %s" % model_file_path)
+
+        # waiting for fix https://github.com/catboost/catboost/issues/696
         Algo = CatBoostClassifier
-        loss_function = "Logloss"
-        if self.params["ml_task"] == MULTICLASS_CLASSIFICATION:
-            loss_function = "MultiClass"
-        elif self.params["ml_task"] == REGRESSION:
-            loss_function = self.params.get("loss_function", "RMSE")
+        if self.params["ml_task"] == REGRESSION:
             Algo = CatBoostRegressor
 
-        self.model = Algo()
-        self.model.load_model(model_file_path)
+        self.model = Algo().load_model(model_file_path)
 
     def get_params(self):
         return {
@@ -130,16 +149,23 @@ class CatBoostAlgorithm(BaseAlgorithm):
 
 classification_params = {
     "learning_rate": [0.005, 0.01, 0.05, 0.1, 0.2],
-    "depth": [4, 5, 6, 7, 8],
-    "rsm": [0.5, 0.6, 0.7, 0.8, 0.9, 1],  # random subspace method
-    "l2_leaf_reg": [1, 3, 5, 7, 10],
+    "depth": [2, 3, 4, 5, 6],
+    "rsm": [0.7, 0.8, 0.9, 1],  # random subspace method
+    "subsample": [0.7, 0.8, 0.9, 1],  # random subspace method
+    "min_data_in_leaf": [1, 5, 10, 15, 20, 30, 50],
+}
+
+classification_default_params = {
+    "learning_rate": 0.1,
+    "depth": 6,
+    "rsm": 0.9,
+    "subsample": 1.0,
+    "min_data_in_leaf": 15,
 }
 
 additional = {
-    "trees_in_step": 10,
-    "train_cant_improve_limit": 5,
-    "min_steps": 5,
-    "max_steps": 500,
+    "max_rounds": 10000,
+    "early_stopping_rounds": 50,
     "max_rows_limit": None,
     "max_cols_limit": None,
 }
@@ -152,6 +178,7 @@ AlgorithmsRegistry.add(
     classification_params,
     required_preprocessing,
     additional,
+    classification_default_params,
 )
 
 AlgorithmsRegistry.add(
@@ -160,18 +187,27 @@ AlgorithmsRegistry.add(
     classification_params,
     required_preprocessing,
     additional,
+    classification_default_params,
 )
 
 regression_params = copy.deepcopy(classification_params)
-regression_params["loss_function"] = ["MAE", "RMSE"]
 
 regression_required_preprocessing = ["missing_values_inputation", "target_scale"]
 
 
+regression_default_params = {
+    "learning_rate": 0.1,
+    "depth": 6,
+    "rsm": 0.9,
+    "subsample": 1.0,
+    "min_data_in_leaf": 15,
+}
+
 AlgorithmsRegistry.add(
     REGRESSION,
     CatBoostAlgorithm,
     regression_params,
     regression_required_preprocessing,
     additional,
+    regression_default_params,
 )
