@@ -6,8 +6,7 @@ from sklearn.tree import DecisionTreeClassifier
 from sklearn.tree import DecisionTreeRegressor
 
 from supervised.algorithms.algorithm import BaseAlgorithm
-from supervised.algorithms.sklearn import SklearnTreesClassifierAlgorithm
-from supervised.algorithms.sklearn import SklearnTreesRegressorAlgorithm
+from supervised.algorithms.sklearn import SklearnAlgorithm
 from supervised.algorithms.registry import AlgorithmsRegistry
 from supervised.algorithms.registry import (
     BINARY_CLASSIFICATION,
@@ -23,7 +22,7 @@ logger.setLevel(LOG_LEVEL)
 from dtreeviz.trees import dtreeviz
 
 
-class DecisionTreeAlgorithm(SklearnTreesClassifierAlgorithm):
+class DecisionTreeAlgorithm(SklearnAlgorithm):
 
     algorithm_name = "Decision Tree"
     algorithm_short_name = "Decision Tree"
@@ -89,7 +88,7 @@ class DecisionTreeAlgorithm(SklearnTreesClassifierAlgorithm):
             logger.info(f"Problem when visualizing decision tree. {str(e)}")
 
 
-class DecisionTreeRegressorAlgorithm(SklearnTreesRegressorAlgorithm):
+class DecisionTreeRegressorAlgorithm(SklearnAlgorithm):
 
     algorithm_name = "Decision Tree"
     algorithm_short_name = "Decision Tree"
@@ -154,6 +153,8 @@ class DecisionTreeRegressorAlgorithm(SklearnTreesRegressorAlgorithm):
 
 dt_params = {"criterion": ["gini", "entropy"], "max_depth": [1, 2, 3, 4]}
 
+classification_default_params = {"criterion": "gini", "max_depth": 3}
+
 additional = {
     "trees_in_step": 1,
     "train_cant_improve_limit": 0,
@@ -173,6 +174,7 @@ AlgorithmsRegistry.add(
     dt_params,
     required_preprocessing,
     additional,
+    classification_default_params,
 )
 
 AlgorithmsRegistry.add(
@@ -181,18 +183,25 @@ AlgorithmsRegistry.add(
     dt_params,
     required_preprocessing,
     additional,
+    classification_default_params,
 )
 
 dt_regression_params = {
-    "criterion": ["mse", "friedman_mse"], # remove "mae" because it slows down a lot https://github.com/scikit-learn/scikit-learn/issues/9626
+    "criterion": [
+        "mse",
+        "friedman_mse",
+    ],  # remove "mae" because it slows down a lot https://github.com/scikit-learn/scikit-learn/issues/9626
     "max_depth": [1, 2, 3, 4],
 }
 regression_required_preprocessing = ["missing_values_inputation", "convert_categorical"]
 
+regression_default_params = {"criterion": "mse", "max_depth": 3}
+
 AlgorithmsRegistry.add(
     REGRESSION,
     DecisionTreeRegressorAlgorithm,
     dt_regression_params,
     regression_required_preprocessing,
     additional,
+    regression_default_params,
 )
