@@ -4,8 +4,10 @@ import sklearn
 from sklearn.ensemble import ExtraTreesClassifier, ExtraTreesRegressor
 
 from supervised.algorithms.algorithm import BaseAlgorithm
-from supervised.algorithms.sklearn import SklearnTreesClassifierAlgorithm
-from supervised.algorithms.sklearn import SklearnTreesRegressorAlgorithm
+from supervised.algorithms.sklearn import (
+    SklearnTreesEnsembleClassifierAlgorithm,
+    SklearnTreesEnsembleRegressorAlgorithm
+)
 from supervised.algorithms.registry import AlgorithmsRegistry
 from supervised.algorithms.registry import (
     BINARY_CLASSIFICATION,
@@ -18,7 +20,7 @@ logger = logging.getLogger(__name__)
 logger.setLevel(LOG_LEVEL)
 
 
-class ExtraTreesAlgorithm(SklearnTreesClassifierAlgorithm):
+class ExtraTreesAlgorithm(SklearnTreesEnsembleClassifierAlgorithm):
 
     algorithm_name = "Extra Trees Classifier"
     algorithm_short_name = "Extra Trees"
@@ -28,14 +30,14 @@ class ExtraTreesAlgorithm(SklearnTreesClassifierAlgorithm):
         logger.debug("ExtraTreesAlgorithm.__init__")
 
         self.library_version = sklearn.__version__
-        self.trees_in_step = additional.get("trees_in_step", 5)
-        self.max_iters = additional.get("max_steps", 3)
+        self.trees_in_step = additional.get("trees_in_step", 100)
+        self.max_steps = additional.get("max_steps", 50)
+        self.early_stopping_rounds = additional.get("early_stopping_rounds", 50)
         self.model = ExtraTreesClassifier(
             n_estimators=self.trees_in_step,
             criterion=params.get("criterion", "gini"),
-            max_features=params.get("max_features", 0.8),
-            min_samples_split=params.get("min_samples_split", 4),
-            min_samples_leaf=params.get("min_samples_leaf", 4),
+            max_features=params.get("max_features", 0.6),
+            min_samples_split=params.get("min_samples_split", 30),
             warm_start=True,
             n_jobs=-1,
             random_state=params.get("seed", 1),
@@ -45,7 +47,7 @@ class ExtraTreesAlgorithm(SklearnTreesClassifierAlgorithm):
         return "extra_trees"
 
 
-class ExtraTreesRegressorAlgorithm(SklearnTreesRegressorAlgorithm):
+class ExtraTreesRegressorAlgorithm(SklearnTreesEnsembleRegressorAlgorithm):
 
     algorithm_name = "Extra Trees Regressor"
     algorithm_short_name = "Extra Trees"
@@ -55,14 +57,14 @@ class ExtraTreesRegressorAlgorithm(SklearnTreesRegressorAlgorithm):
         logger.debug("ExtraTreesRegressorAlgorithm.__init__")
 
         self.library_version = sklearn.__version__
-        self.trees_in_step = additional.get("trees_in_step", 5)
-        self.max_iters = additional.get("max_steps", 3)
+        self.trees_in_step = regression_additional.get("trees_in_step", 100)
+        self.max_steps = regression_additional.get("max_steps", 50)
+        self.early_stopping_rounds = regression_additional.get("early_stopping_rounds", 50)
         self.model = ExtraTreesRegressor(
             n_estimators=self.trees_in_step,
             criterion=params.get("criterion", "mse"),
             max_features=params.get("max_features", 0.8),
             min_samples_split=params.get("min_samples_split", 4),
-            min_samples_leaf=params.get("min_samples_leaf", 4),
             warm_start=True,
             n_jobs=-1,
             random_state=params.get("seed", 1),
@@ -75,16 +77,20 @@ class ExtraTreesRegressorAlgorithm(SklearnTreesRegressorAlgorithm):
 # For binary classification target should be 0, 1. There should be no NaNs in target.
 et_params = {
     "criterion": ["gini", "entropy"],
-    "max_features": [0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 0.95, 1.0],
-    "min_samples_split": [2, 4, 6, 8, 10, 15, 20, 30, 40, 50],
-    "min_samples_leaf": range(1, 21),
+    "max_features": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
+    "min_samples_split": [10, 20, 30, 40, 50],
+}
+
+classification_default_params = {
+    "criterion": "gini",
+    "max_features": 0.6,
+    "min_samples_split": 30,
 }
 
 additional = {
-    "trees_in_step": 10,
-    "train_cant_improve_limit": 5,
-    "min_steps": 5,
-    "max_steps": 500,
+    "trees_in_step": 100,
+    "max_steps": 50,
+    "early_stopping_rounds": 50,
     "max_rows_limit": None,
     "max_cols_limit": None,
 }
@@ -100,6 +106,7 @@ AlgorithmsRegistry.add(
     et_params,
     required_preprocessing,
     additional,
+    classification_default_params,
 )
 
 AlgorithmsRegistry.add(
@@ -108,6 +115,7 @@ AlgorithmsRegistry.add(
     et_params,
     required_preprocessing,
     additional,
+    classification_default_params,
 )
 
 
@@ -116,16 +124,23 @@ AlgorithmsRegistry.add(
 #
 
 regression_et_params = {
-    "criterion": ["mse"], # remove "mae" because it slows down a lot https://github.com/scikit-learn/scikit-learn/issues/9626
-    "max_features": [0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 0.95, 1.0],
-    "min_samples_split": [2, 4, 6, 8, 10, 15, 20, 30, 40, 50],
-    "min_samples_leaf": range(1, 21),
+    "criterion": [
+        "mse"
+    ],  # remove "mae" because it slows down a lot https://github.com/scikit-learn/scikit-learn/issues/9626
+    "max_features": [0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
+    "min_samples_split": [10, 20, 30, 40, 50],
+}
+
+regression_default_params = {
+    "criterion": "mse",
+    "max_features": 0.6,
+    "min_samples_split": 30,
 }
 
 regression_additional = {
-    "trees_in_step": 10,
-    "train_cant_improve_limit": 5,
-    "max_steps": 500,
+    "trees_in_step": 100,
+    "max_steps": 50,
+    "early_stopping_rounds": 50,
     "max_rows_limit": None,
     "max_cols_limit": None,
 }
@@ -141,4 +156,5 @@ AlgorithmsRegistry.add(
     regression_et_params,
     regression_required_preprocessing,
     regression_additional,
+    regression_default_params,
 )
