@@ -29,18 +29,21 @@ class LightgbmAlgorithm(BaseAlgorithm):
         super(LightgbmAlgorithm, self).__init__(params)
         self.library_version = lgb.__version__
 
-        self.rounds = additional.get("trees_in_step", 50)
-        self.max_iters = additional.get("max_steps", 500)
+        self.explain_level = params.get("explain_level", 0)
+        self.rounds = additional.get("max_rounds", 10000) 
+        self.max_iters = 1 
+        self.early_stopping_rounds = additional.get("early_stopping_rounds", 50)
+    
         self.learner_params = {
             "boosting_type": "gbdt",
             "objective": self.params.get("objective", "binary"),
             "metric": self.params.get("metric", "binary_logloss"),
             "num_threads": multiprocessing.cpu_count(),
-            "num_leaves": self.params.get("num_leaves", 16),
-            "learning_rate": self.params.get("learning_rate", 0.01),
-            "feature_fraction": self.params.get("feature_fraction", 0.7),
-            "bagging_fraction": self.params.get("bagging_fraction", 0.7),
-            "bagging_freq": self.params.get("bagging_freq", 1),
+            "num_leaves": self.params.get("num_leaves", 31),
+            "learning_rate": self.params.get("learning_rate", 0.1),
+            "feature_fraction": self.params.get("feature_fraction", 1.0),
+            "bagging_fraction": self.params.get("bagging_fraction", 1.0),
+            "min_data_in_leaf": self.params.get("min_data_in_leaf", 20),
             "verbose": -1,
             "seed": self.params.get("seed", 1),
         }
@@ -55,14 +58,48 @@ class LightgbmAlgorithm(BaseAlgorithm):
     def update(self, update_params):
         pass
 
-    def fit(self, X, y):
+    def fit(self, X, y, X_validation=None, y_validation=None, log_to_file=None):
+
         lgb_train = lgb.Dataset(X, y)
-        self.model = lgb.train(
-            self.learner_params,
-            lgb_train,
-            num_boost_round=self.rounds,
-            init_model=self.model,
-        )
+        if self.early_stopping_rounds == 0:
+            self.model = lgb.train(
+                self.learner_params,
+                lgb_train,
+                num_boost_round=self.rounds,
+                init_model=self.model,
+            )
+        else:
+            valid_sets = None 
+            valid_names = None 
+            esr = None
+            if X_validation is not None and y_validation is not None:
+                valid_sets = [lgb_train, lgb.Dataset(X_validation, y_validation)]
+                valid_names = ["train", "validation"]
+                esr = self.early_stopping_rounds
+            evals_result = {}
+            self.model = lgb.train(
+                self.learner_params,
+                lgb_train,
+                num_boost_round=self.rounds,
+                init_model=self.model,
+                valid_sets=valid_sets,
+                valid_names=valid_names,
+                early_stopping_rounds=esr,
+                evals_result=evals_result,
+                verbose_eval=False,
+            )
+
+            if log_to_file is not None:
+                metric_name = list(evals_result["train"].keys())[0]
+                result = pd.DataFrame(
+                    {
+                        "iteration": range(len(evals_result["train"][metric_name])),
+                        "train": evals_result["train"][metric_name],
+                        "validation": evals_result["validation"][metric_name],
+                    }
+                )
+                result.to_csv(log_to_file, index=False, header=False)
+            
 
     def predict(self, X):
         return self.model.predict(X)
@@ -101,20 +138,28 @@ class LightgbmAlgorithm(BaseAlgorithm):
 
 lgbm_bin_params = {
     "objective": ["binary"],
-    "metric": ["binary_logloss", "auc"],
-    "num_leaves": [32, 64, 128, 256, 512],
-    "learning_rate": [0.01, 0.025, 0.05, 0.075, 0.1],
-    "feature_fraction": [0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
-    "bagging_fraction": [0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
-    "bagging_freq": [0, 1, 2, 3, 4, 5],
+    "metric": ["binary_logloss"],
+    "num_leaves": [3, 7, 15, 31], #, 63, 127],
+    "learning_rate": [0.05, 0.075, 0.1, 0.15],
+    "feature_fraction": [0.8, 0.9, 1.0],
+    "bagging_fraction": [0.8, 0.9, 1.0],
+    "min_data_in_leaf": [5, 10, 15, 20, 30, 50]
+}
+
+classification_bin_default_params = {
+    "objective": "binary",
+    "metric": "binary_logloss",
+    "num_leaves": 31,
+    "learning_rate": 0.1,
+    "feature_fraction": 0.9,
+    "bagging_fraction": 0.9,
+    "min_data_in_leaf": 10
 }
 
 
 additional = {
-    "trees_in_step": 10,
-    "train_cant_improve_limit": 5,
-    "min_steps": 5,
-    "max_steps": 500,
+    "max_rounds": 10000,
+    "early_stopping_rounds": 50,
     "max_rows_limit": None,
     "max_cols_limit": None,
 }
@@ -125,15 +170,23 @@ required_preprocessing = [
     "target_as_integer",
 ]
 
-
 lgbm_multi_params = copy.deepcopy(lgbm_bin_params)
 lgbm_multi_params["objective"] = ["multiclass"]
-lgbm_multi_params["metric"] = ["multi_logloss", "multi_error"]
+lgbm_multi_params["metric"] = ["multi_logloss"]
 
+classification_multi_default_params = {
+    "objective": "multiclass",
+    "metric": "multi_logloss",
+    "num_leaves": 31,
+    "learning_rate": 0.1,
+    "feature_fraction": 0.9,
+    "bagging_fraction": 0.9,
+    "min_data_in_leaf": 10
+}
 
 lgbr_params = copy.deepcopy(lgbm_bin_params)
 lgbr_params["objective"] = ["regression"]
-lgbr_params["metric"] = ["l1", "l2"]
+lgbr_params["metric"] = ["l2"]
 
 AlgorithmsRegistry.add(
     BINARY_CLASSIFICATION,
@@ -141,6 +194,7 @@ AlgorithmsRegistry.add(
     lgbm_bin_params,
     required_preprocessing,
     additional,
+    classification_bin_default_params,
 )
 
 AlgorithmsRegistry.add(
@@ -149,6 +203,7 @@ AlgorithmsRegistry.add(
     lgbm_multi_params,
     required_preprocessing,
     additional,
+    classification_multi_default_params,
 )
 
 regression_required_preprocessing = [
@@ -158,10 +213,21 @@ regression_required_preprocessing = [
 ]
 
 
+regression_default_params = {
+    "objective": "regression",
+    "metric": "l2",
+    "num_leaves": 15,
+    "learning_rate": 0.1,
+    "feature_fraction": 0.9,
+    "bagging_fraction": 0.9,
+    "min_data_in_leaf": 10
+}
+
 AlgorithmsRegistry.add(
     REGRESSION,
     LightgbmAlgorithm,
     lgbr_params,
     regression_required_preprocessing,
     additional,
+    regression_default_params,
 )
