@@ -26,6 +26,10 @@ class XgbAlgorithmException(Exception):
         logger.error(message)
 
 
+def time_constraint(env):
+    #print("time constraint")
+    pass
+
 class XgbAlgorithm(BaseAlgorithm):
     """
     This is a wrapper over xgboost algorithm.
@@ -38,8 +42,11 @@ class XgbAlgorithm(BaseAlgorithm):
         super(XgbAlgorithm, self).__init__(params)
         self.library_version = xgb.__version__
 
-        self.boosting_rounds = additional.get("trees_in_step", 50)
-        self.max_iters = additional.get("max_steps", 500)
+        self.explain_level = params.get("explain_level", 0)
+        self.boosting_rounds = additional.get("max_rounds", 10000) 
+        self.max_iters = 1
+        self.early_stopping_rounds = additional.get("early_stopping_rounds", 50)
+
         self.learner_params = {
             "tree_method": "hist",
             "booster": "gbtree",
@@ -60,14 +67,44 @@ class XgbAlgorithm(BaseAlgorithm):
         if "num_class" in self.params:  # multiclass classification
             self.learner_params["num_class"] = self.params.get("num_class")
 
+        self.best_ntree_limit = 0
         logger.debug("XgbLearner __init__")
 
-    def fit(self, X, y):
+    def fit(self, X, y, X_validation=None, y_validation=None, log_to_file=None):
         dtrain = xgb.DMatrix(X, label=y, missing=np.NaN)
+        dvalidation = xgb.DMatrix(X_validation, label=y_validation, missing=np.NaN)
+        evals_result = {}
+
+        evals = [] 
+        esr = None
+        if X_validation is not None and y_validation is not None:
+            evals = [(dtrain, "train"), (dvalidation, "validation")]
+            esr = self.early_stopping_rounds
         self.model = xgb.train(
-            self.learner_params, dtrain, self.boosting_rounds, xgb_model=self.model
+            self.learner_params,
+            dtrain,
+            self.boosting_rounds,
+            evals=evals,
+            early_stopping_rounds=esr,
+            evals_result=evals_result,
+            verbose_eval=False,
+            #callbacks=[time_constraint] # callback slows down by factor ~8
         )
 
+        if log_to_file is not None:
+            metric_name = list(evals_result["train"].keys())[0]
+            result = pd.DataFrame(
+                {
+                    "iteration": range(len(evals_result["train"][metric_name])),
+                    "train": evals_result["train"][metric_name],
+                    "validation": evals_result["validation"][metric_name],
+                }
+            )
+            result.to_csv(log_to_file, index=False, header=False)
+
+        # save best_ntree_limit because of:
+        # https://github.com/dmlc/xgboost/issues/805
+        self.best_ntree_limit = self.model.best_ntree_limit
         # fix high memory consumption in xgboost,
         # waiting for release with fix
         # https://github.com/dmlc/xgboost/issues/5474
@@ -80,8 +117,9 @@ class XgbAlgorithm(BaseAlgorithm):
     def predict(self, X):
         if self.model is None:
             raise XgbAlgorithmException("Xgboost model is None")
+
         dtrain = xgb.DMatrix(X, missing=np.NaN)
-        a = self.model.predict(dtrain)
+        a = self.model.predict(dtrain, ntree_limit=self.best_ntree_limit)
         return a
 
     def copy(self):
@@ -103,6 +141,7 @@ class XgbAlgorithm(BaseAlgorithm):
             "algorithm_short_name": self.algorithm_short_name,
             "uid": self.uid,
             "params": self.params,
+            "best_ntree_limit": self.best_ntree_limit,
         }
 
     def set_params(self, json_desc):
@@ -113,6 +152,7 @@ class XgbAlgorithm(BaseAlgorithm):
         )
         self.uid = json_desc.get("uid", self.uid)
         self.params = json_desc.get("params", self.params)
+        self.best_ntree_limit = json_desc.get("best_ntree_limit", self.best_ntree_limit)
 
     def file_extension(self):
         return "xgboost"
@@ -121,7 +161,7 @@ class XgbAlgorithm(BaseAlgorithm):
 # For binary classification target should be 0, 1. There should be no NaNs in target.
 xgb_bin_class_params = {
     "objective": ["binary:logistic"],
-    "eval_metric": ["auc", "logloss"],
+    "eval_metric": ["logloss"],
     "eta": [0.01, 0.025, 0.05, 0.075, 0.1, 0.15],
     "max_depth": [1, 2, 3, 4, 5, 6, 7, 8, 9],
     "min_child_weight": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
@@ -129,9 +169,19 @@ xgb_bin_class_params = {
     "colsample_bytree": [0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0],
 }
 
+classification_bin_default_params = {
+    "objective": "binary:logistic",
+    "eval_metric": "logloss",
+    "eta": 0.1,
+    "max_depth": 6,
+    "min_child_weight": 1,
+    "subsample": 1.0,
+    "colsample_bytree": 1.0,
+}
+
 xgb_regression_params = dict(xgb_bin_class_params)
 xgb_regression_params["objective"] = ["reg:squarederror"]
-xgb_regression_params["eval_metric"] = ["rmse", "rmsle", "mae"]
+xgb_regression_params["eval_metric"] = ["rmse"]
 xgb_regression_params["max_depth"] = [1, 2, 3, 4]
 
 
@@ -139,12 +189,30 @@ xgb_multi_class_params = dict(xgb_bin_class_params)
 xgb_multi_class_params["objective"] = ["multi:softprob"]
 xgb_multi_class_params["eval_metric"] = ["mlogloss"]
 
+classification_multi_default_params = {
+    "objective": "multi:softprob",
+    "eval_metric": "mlogloss",
+    "eta": 0.1,
+    "max_depth": 6,
+    "min_child_weight": 1,
+    "subsample": 1.0,
+    "colsample_bytree": 1.0,
+}
+
+
+regression_default_params = {
+    "objective": "reg:squarederror",
+    "eval_metric": "rmse",
+    "eta": 0.1,
+    "max_depth": 4,
+    "min_child_weight": 1,
+    "subsample": 1.0,
+    "colsample_bytree": 1.0,
+}
 
 additional = {
-    "trees_in_step": 10,
-    "train_cant_improve_limit": 5,
-    "min_steps": 5,
-    "max_steps": 500,
+    "max_rounds": 10000,
+    "early_stopping_rounds": 50,
     "max_rows_limit": None,
     "max_cols_limit": None,
 }
@@ -160,6 +228,7 @@ AlgorithmsRegistry.add(
     xgb_bin_class_params,
     required_preprocessing,
     additional,
+    classification_bin_default_params,
 )
 
 AlgorithmsRegistry.add(
@@ -168,6 +237,7 @@ AlgorithmsRegistry.add(
     xgb_multi_class_params,
     required_preprocessing,
     additional,
+    classification_multi_default_params,
 )
 
 regression_required_preprocessing = [
@@ -183,4 +253,5 @@ AlgorithmsRegistry.add(
     xgb_regression_params,
     regression_required_preprocessing,
     additional,
+    regression_default_params,
 )
