@@ -10,7 +10,9 @@ import logging
 from supervised.model_framework import ModelFramework
 from supervised.callbacks.early_stopping import EarlyStopping
 from supervised.callbacks.metric_logger import MetricLogger
-from supervised.callbacks.time_constraint import TimeConstraint
+from supervised.callbacks.learner_time_constraint import LearnerTimeConstraint
+from supervised.callbacks.total_time_constraint import TotalTimeConstraint
+
 from supervised.utils.metric import Metric
 from supervised.algorithms.registry import AlgorithmsRegistry
 from supervised.algorithms.registry import (
@@ -32,7 +34,7 @@ logger = logging.getLogger(__name__)
 logger.setLevel(LOG_LEVEL)
 
 from supervised.exceptions import AutoMLException
-
+from supervised.tuner.data_info import DataInfo
 import gc
 from supervised.utils.config import mem
 
@@ -58,13 +60,16 @@ class AutoML:
             "LightGBM",
             "Xgboost",
             "CatBoost",
+            "Neural Network",
+            "Nearest Neighbors",
         ],
-        tuning_mode="Sport",
+        tuning_mode="Normal",
         train_ensemble=True,
+        stack=True,
         optimize_metric=None,
         validation={
             "validation_type": "kfold",
-            "k_folds": 5,
+            "k_folds": 10,
             "shuffle": True,
             "stratify": True,
         },
@@ -102,7 +107,9 @@ class AutoML:
         
         You can also set how many models will be trained with `set_advanced` method.
         
-        :param train_ensemble: If true then at the end of models training the ensemble will be created.
+        :param train_ensemble: If true then at the end of models training the ensemble will be created. (Default is `True`)
+
+        :param stack: If true then stacked models will be created. Stack level is 1. (Default is `True`)
         
         :param optimize_metric: The metric to be optimized. (not implemented yet, please left `None`)
         
@@ -138,6 +145,7 @@ class AutoML:
         self._time_limit = 1
 
         self._train_ensemble = train_ensemble
+        self._stack = stack
         self._models = []  # instances of iterative learner framework or ensemble
 
         # it is instance of model framework or ensemble
@@ -165,10 +173,20 @@ class AutoML:
 
         self._data_info = None
         self._model_paths = []
-
-        self._results_path = results_path
-        self._set_results_dir()
+        self._stacked_models = None
         self._explain_level = explain_level
+        self._results_path = results_path
+        self._fit_level = None
+        self._time_spend = {}
+        self._start_time = time.time()  # it will be updated in `fit` method
+
+        if self._validation["validation_type"] != "kfold":
+            # stack only available of k-fold validation
+            self._stack = False
+
+        # this should be last in the constrcutor
+        # in case there is a dir, it might load models
+        self._set_results_dir()
 
     def set_tuning_mode(self, mode="Normal"):
         if mode == "Sport":
@@ -255,17 +273,24 @@ class AutoML:
             self._model_paths = params["saved"]
             self._ml_task = params["ml_task"]
             self._optimize_metric = params["optimize_metric"]
+            stacked_models = params.get("stacked")
 
             models_map = {}
             for model_path in self._model_paths:
-                if model_path.endswith("ensemble"):
+                if model_path.endswith("Ensemble") or model_path.endswith("Ensemble_Stacked"):
                     ens = Ensemble.load(model_path, models_map)
+                    self._models += [ens]
                     models_map[ens.get_name()] = ens
                 else:
                     m = ModelFramework.load(model_path)
                     self._models += [m]
                     models_map[m.get_name()] = m
 
+            if stacked_models is not None:
+                self._stacked_models = []
+                for stacked_model_name in stacked_models:
+                    self._stacked_models += [models_map[stacked_model_name]]
+
             best_model_name = None
             with open(os.path.join(self._results_path, "best_model.txt"), "r") as fin:
                 best_model_name = fin.read()
@@ -277,40 +302,6 @@ class AutoML:
         except Exception as e:
             raise AutoMLException(f"Cannot load AutoML directory. {str(e)}")
 
-    def _estimate_training_times(self):
-
-        algo_cnt = len(self._algorithms)
-        if "Baseline" in self._algorithms:
-            algo_cnt -= 1
-        if "Linear" in self._algorithms:
-            algo_cnt -= 1
-
-        self._estimated_models_to_check = algo_cnt * self._start_random_models
-        if self._estimated_models_to_check > self._top_models_to_improve:
-            self._estimated_models_to_check += (
-                self._top_models_to_improve * self._hill_climbing_steps * 2
-            )
-
-        if "Baseline" in self._algorithms:
-            self._estimated_models_to_check += 1
-        if "Linear" in self._algorithms:
-            self._estimated_models_to_check += 1
-
-        if self._model_time_limit is not None:
-            k = self._validation.get("k_folds", 1.0)
-            self._time_limit = self._model_time_limit / k
-        elif self._total_time_limit is not None:
-            # set time limit for single model training
-            # the 0.85 is safe scale factor, to not exceed time limit
-            # scaling is added because number of models to be trained are estimate
-            k = self._validation.get("k_folds", 1.0)
-            self._time_limit = (
-                self._total_time_limit * 0.85 / self._estimated_models_to_check / k
-            )
-        print(
-            f"AutoML will try to check about {int(self._estimated_models_to_check)} model{'s' if int(self._estimated_models_to_check)>1 else ''}"
-        )
-
     def get_leaderboard(self):
         ldb = {
             "name": [],
@@ -333,7 +324,7 @@ class AutoML:
         self._models += [model]
         self.verbose_print(
             "{} final {} {} time {} seconds".format(
-                model.get_type(),
+                model.get_name(),
                 self._optimize_metric,
                 model.get_final_loss(),
                 np.round(model.get_train_time(), 2),
@@ -341,27 +332,105 @@ class AutoML:
         )
         self.log_train_time(model.get_type(), model.get_train_time())
 
+    def _get_learner_time_limit(self, model_type):
+
+        logger.debug(
+            f"Fit level: {self._fit_level}, model type: {model_type}. "
+            + f"Time spend: {json.dumps(self._time_spend, indent=4)}"
+        )
+
+        if self._model_time_limit is not None:
+            k = self._validation.get("k_folds", 1.0)
+            return self._model_time_limit / k
+
+        if self._fit_level == "simple_algorithms":
+            return None
+        if self._fit_level == "default_algorithms":
+            return None
+
+        tune_algorithms = [
+            a
+            for a in self._algorithms
+            if a not in ["Baseline", "Linear", "Decision Tree", "Nearest Neighbors"]
+        ]
+        tune_algs_cnt = len(tune_algorithms)
+        if tune_algs_cnt == 0:
+            return None
+
+        time_elapsed = time.time() - self._start_time
+        time_left = self._total_time_limit - time_elapsed
+
+        k_folds = self._validation.get("k_folds", 1.0)
+
+        if self._fit_level == "not_so_random":
+            tt = (
+                self._total_time_limit
+                - self._time_spend["simple_algorithms"]
+                - self._time_spend["default_algorithms"]
+            )
+            if self._stack:
+                tt *= (
+                    0.6
+                )  # leave some time for stacking (approx. 40% for stacking of time left)
+            tt /= 2.0  # leave some time for hill-climbing
+            tt /= tune_algs_cnt  # give time equally for each algorithm
+            tt /= k_folds  # time is per learner (per fold)
+            return tt
+
+        if self._fit_level == "hill_climbing":
+            tt = (
+                self._total_time_limit
+                - self._time_spend["simple_algorithms"]
+                - self._time_spend["default_algorithms"]
+                - self._time_spend["not_so_random"]
+            )
+            if self._stack:
+                tt *= (
+                    0.4
+                )  # leave some time for stacking (approx. 60% for stacking of time left)
+            tt /= tune_algs_cnt  # give time equally for each algorithm
+            tt /= k_folds  # time is per learner (per fold)
+            return tt
+
+        if self._stack and self._fit_level == "stack":
+            tt = time_left
+            tt /= tune_algs_cnt  # give time equally for each algorithm
+            tt /= k_folds  # time is per learner (per fold)
+            return tt
+
     def train_model(self, params):
 
         model_path = os.path.join(self._results_path, params["name"])
-
         early_stop = EarlyStopping(
             {"metric": {"name": self._optimize_metric}, "log_to_dir": model_path}
         )
-        min_steps = params["additional"].get("min_steps")
-        
-        time_constraint = TimeConstraint(
+
+        learner_time_constraint = LearnerTimeConstraint(
             {
-                "train_seconds_time_limit": self._time_limit,
-                "min_steps": min_steps,
-                "total_time_limit": self._total_time_limit if self._model_time_limit is None else None,
-                "total_time_start": self._start_time
+                "learner_time_limit": self._get_learner_time_limit(
+                    params["learner"]["model_type"]
+                ),  # self._time_limit,
+                "min_steps": params["additional"].get("min_steps"),
             }
         )
-        mf = ModelFramework(params, callbacks=[early_stop, time_constraint])
+
+        total_time_constraint = TotalTimeConstraint(
+            {
+                "total_time_limit": self._total_time_limit
+                if self._model_time_limit is None
+                else None,
+                "total_time_start": self._start_time,
+            }
+        )
+
+        mf = ModelFramework(
+            params,
+            callbacks=[early_stop, learner_time_constraint, total_time_constraint],
+        )
 
         if self._enough_time_to_train(mf.get_type()):
 
+            #self.verbose_print(params["name"] + " training start ...")
             logger.info(
                 f"Train model #{len(self._models)+1} / Model name: {params['name']}"
             )
@@ -381,9 +450,7 @@ class AutoML:
             # save the best one in the case the training will be interrupted
             self.select_and_save_best()
         else:
-            logger.info(
-                f"Cannot check more models of {mf.get_type()} because of time constraint"
-            )
+            logger.info(f"Cannot train {mf.get_type()} because of time constraint")
         # self._progress_bar.update(1)
 
     def verbose_print(self, msg):
@@ -406,10 +473,7 @@ class AutoML:
         if self._total_time_limit is None:
             return True
 
-        total_time_spend = 0.0
-        for k, v in self._models_train_time.items():
-            total_time_spend += np.sum(v)
-
+        total_time_spend = time.time() - self._start_time
         # no time left, do not train more models, sorry ...
         time_left = self._total_time_limit - total_time_spend
         if time_left < 0:
@@ -420,6 +484,52 @@ class AutoML:
         if time_left > 0 and model_type not in self._models_train_time:
             return True
 
+        # check the fit level type
+        # we dont want to spend too much time on one level
+
+        if self._fit_level == "not_so_random":
+
+            time_should_use = (
+                self._total_time_limit
+                - self._time_spend["simple_algorithms"]
+                - self._time_spend["default_algorithms"]
+            )
+            if self._stack:
+                time_should_use *= 0.6  # leave time for stacking
+            if self._hill_climbing_steps > 0:
+                time_should_use /= 2.0  # leave time for hill-climbing
+
+            if (
+                total_time_spend
+                > time_should_use
+                + self._time_spend["simple_algorithms"]
+                + self._time_spend["default_algorithms"]
+            ):
+                return False
+
+        ##################
+        # hill climbing check
+
+        if self._fit_level == "hill_climbing":
+
+            time_should_use = (
+                self._total_time_limit
+                - self._time_spend["simple_algorithms"]
+                - self._time_spend["default_algorithms"]
+                - self._time_spend["not_so_random"]
+            )
+            if self._stack:
+                time_should_use *= 0.4  # leave time for stacking
+
+            if (
+                total_time_spend
+                > time_should_use
+                + self._time_spend["simple_algorithms"]
+                + self._time_spend["default_algorithms"]
+                + self._time_spend["not_so_random"]
+            ):
+                return False
+
         model_total_time_spend = (
             0
             if model_type not in self._models_train_time
@@ -432,9 +542,11 @@ class AutoML:
         )
 
         algo_cnt = float(len(self._algorithms))
-        for a in ["Baseline", "Decision Tree"]:
+        for a in ["Baseline", "Decision Tree", "Linear", "Nearest Neighbors"]:
             if a in self._algorithms:
                 algo_cnt -= 1.0
+        if algo_cnt < 1.0:
+            algo_cnt = 1.0
 
         model_time_left = time_left / algo_cnt
         if model_mean_time_spend <= model_time_left:
@@ -442,14 +554,18 @@ class AutoML:
 
         return False
 
-    def ensemble_step(self):
+    def ensemble_step(self, is_stacked=False):
         if self._train_ensemble and len(self._models) > 1:
-            self.ensemble = Ensemble(self._optimize_metric, self._ml_task)
+            self.ensemble = Ensemble(
+                self._optimize_metric, self._ml_task, is_stacked=is_stacked
+            )
             oofs, target = self.ensemble.get_oof_matrix(self._models)
             self.ensemble.fit(oofs, target)
             self.keep_model(self.ensemble)
 
-            ensemble_path = os.path.join(self._results_path, "ensemble")
+            ensemble_path = os.path.join(
+                self._results_path, "Ensemble_Stacked" if is_stacked else "Ensemble"
+            )
             try:
                 os.mkdir(ensemble_path)
             except Exception as e:
@@ -459,6 +575,125 @@ class AutoML:
             # save the best one in the case the training will be interrupted
             self.select_and_save_best()
 
+    def can_we_stack_them(self, y):
+        # if multiclass and too many classes then No
+        return True
+
+    def get_stacked_data(self, X, mode="training"):
+        # mode can be `training` or `predict`
+        if self._stacked_models is None:
+            return X
+        all_oofs = []
+        for m in self._stacked_models:
+            oof = None
+            if mode == "training":
+                oof = m.get_out_of_folds()
+            else:
+                oof = m.predict(X)
+                if self._ml_task == BINARY_CLASSIFICATION:
+                    cols = [f for f in oof.columns if "prediction" in f]
+                    if len(cols) == 2:
+                        oof = pd.DataFrame({"prediction": oof[cols[1]]})
+
+            cols = [f for f in oof.columns if "prediction" in f]
+            oof = oof[cols]
+            oof.columns = [f"{m.get_name()}_{c}" for c in cols]
+            all_oofs += [oof]
+
+        X_stacked = pd.concat(all_oofs + [X], axis=1)
+
+        return X_stacked
+
+    def stack_models(self):
+
+        if self._stacked_models is not None:
+            return
+
+        ldb = self.get_leaderboard()
+        ldb = ldb.sort_values(by="metric_value", ascending=True)
+
+        models_map = {m.get_name(): m for m in self._models if not m._is_stacked}
+        self._stacked_models = []
+        models_limit = 10
+
+        for model_type in np.unique(ldb.model_type):
+            if model_type in ["Baseline"]:
+                continue
+            ds = ldb[ldb.model_type == model_type].copy()
+            ds.sort_values(by="metric_value", inplace=True)
+            
+            for n in list(ds.name.iloc[:models_limit].values):
+                self._stacked_models += [models_map[n]]
+
+        scores = [m.get_final_loss() for m in self._stacked_models]
+        self._stacked_models = [
+            self._stacked_models[i] for i in np.argsort(scores).tolist()
+        ]
+
+    def stacked_ensemble_step(self):
+        #print("Stacked models ....")
+        # do we have enough models?
+        if len(self._models) < 5:
+            return
+        # do we have time?
+        #if self._total_time_limit is not None:
+        #    time_left = self._total_time_limit - (time.time() - self._start_time)
+        #    # we need at least 60 seconds to do anything
+        #    if time_left < 60:
+        #        return
+
+        # read X directly from parquet
+        X = pd.read_parquet(self._X_train_path)
+
+        self.stack_models()
+
+        org_columns = X.columns.tolist()
+        X_stacked = self.get_stacked_data(X)
+        new_columns = X_stacked.columns.tolist()
+        added_columns = [c for c in new_columns if c not in org_columns]
+
+        # save stacked data
+        X_train_stacked_path = os.path.join(
+            self._results_path, "X_train_stacked.parquet"
+        )
+        X_stacked.to_parquet(X_train_stacked_path, index=False)
+
+        # resue old params
+        for m in self._stacked_models:
+            #print(m.get_type())
+            # use only Xgboost, LightGBM and CatBoost as stacked models 
+            if m.get_type() not in [
+                "Xgboost",
+                "LightGBM",
+                "CatBoost"
+            ]:
+                continue
+
+            params = copy.deepcopy(m.params)    
+            params["validation"]["X_train_path"] = X_train_stacked_path
+
+            params["name"] = params["name"] + "_Stacked"
+            params["is_stacked"] = True
+            #print(params)
+
+            if "model_architecture_json" in params["learner"]:
+                # the new model will be created with wider input size
+                del params["learner"]["model_architecture_json"]
+
+            if self._ml_task == REGRESSION:
+                # scale added predictions in regression if the target was scaled (in the case of NN)
+                target_preprocessing = params["preprocessing"]["target_preprocessing"]
+                scale = None
+                if "scale_log_and_normal" in target_preprocessing:
+                    scale = "scale_log_and_normal"
+                elif "scale_normal" in target_preprocessing:
+                    scale = "scale_normal"
+                if scale is not None:
+                    for col in added_columns:
+                        params["preprocessing"]["columns_preprocessing"][col] = [scale]
+            
+            self.train_model(params)
+
     def _set_ml_task(self, y):
         """ Set and validate the ML task.
         
@@ -535,8 +770,8 @@ class AutoML:
                 self._optimize_metric = self._user_set_optimize_metric
         elif self._ml_task == REGRESSION:
             if self._user_set_optimize_metric is None:
-                self._optimize_metric = "mse"
-            elif self._user_set_optimize_metric not in ["mse"]:
+                self._optimize_metric = "rmse"
+            elif self._user_set_optimize_metric not in ["rmse"]:
                 raise AutoMLException(
                     "Metric {} is not allowed in ML task: {}".format(
                         self._user_set_optimize_metric, self._ml_task
@@ -603,25 +838,33 @@ class AutoML:
         self._validation["y_train_path"] = self._y_train_path
         self._validation["results_path"] = self._results_path
 
+        columns_and_target_info = DataInfo.compute(X_train, y_train, self._ml_task)
+
         self._data_info = {
             "columns": X_train.columns.tolist(),
             "rows": X_train.shape[0],
             "cols": X_train.shape[1],
             "target_is_numeric": pd.api.types.is_numeric_dtype(y_train),
+            "columns_info": columns_and_target_info["columns_info"],
+            "target_info": columns_and_target_info["target_info"],
         }
+        if columns_and_target_info.get("num_class") is not None:
+            self._data_info["num_class"] = columns_and_target_info["num_class"]
         data_info_path = os.path.join(self._results_path, "data_info.json")
         with open(data_info_path, "w") as fout:
             fout.write(json.dumps(self._data_info, indent=4))
 
-    def _del_data_variables(self, X_train, y_train):
+        self._drop_data_variables(X_train)
+
+    def _drop_data_variables(self, X_train):
 
         X_train.drop(X_train.columns, axis=1, inplace=True)
 
     def _load_data_variables(self, X_train):
-        X = pd.read_parquet(self._X_train_path)
-
-        for c in X.columns:
-            X_train.insert(loc=X_train.shape[1], column=c, value=X[c])
+        if X_train.shape[1] == 0:
+            X = pd.read_parquet(self._X_train_path)
+            for c in X.columns:
+                X_train.insert(loc=X_train.shape[1], column=c, value=X[c])
 
         os.remove(self._X_train_path)
         os.remove(self._y_train_path)
@@ -639,16 +882,19 @@ class AutoML:
         """
         try:
 
-            if self._best_model is not None:
-                print("Best model is already set, no need to run fit. Skipping ...")
-                return
+            #if self._best_model is not None:
+            #    print("Best model is already set, no need to run fit. Skipping ...")
+            #    return
 
             self._start_time = time.time()
+
             if not isinstance(X_train, pd.DataFrame):
                 raise AutoMLException(
                     "AutoML needs X_train matrix to be a Pandas DataFrame"
                 )
 
+            self._set_ml_task(y_train)
+
             if X_train is not None:
                 X_train = X_train.copy(deep=False)
 
@@ -657,10 +903,9 @@ class AutoML:
             )
             self._save_data(X_train, y_train, X_validation, y_validation)
 
-            self._set_ml_task(y_train)
             self._set_algorithms()
             self._set_metric()
-            self._estimate_training_times()
+            # self._estimate_training_times()
 
             if self._ml_task in [BINARY_CLASSIFICATION, MULTICLASS_CLASSIFICATION]:
                 self._check_imbalanced(y_train)
@@ -671,40 +916,80 @@ class AutoML:
                 self._ml_task,
                 self._validation,
                 self._explain_level,
+                self._data_info,
                 self._seed,
             )
+            self.tuner = tuner
+            self._time_spend = {}
+            self._time_start = {}
 
-            # not so random step
-            generated_params = tuner.get_not_so_random_params(X_train, y_train)
-            self._del_data_variables(X_train, y_train)
+            
+            # 1. Check simple algorithms
+            self._fit_level = "simple_algorithms"
+            start = time.time()
+            self._time_start[self._fit_level] = start
+            for params in tuner.simple_algorithms_params():
+                self.train_model(params)
+            self._time_spend["simple_algorithms"] = np.round(time.time() - start, 2)
 
-            # Shuffle generated params
-            # do not shuffle Baseline, Linear and Decision Trees
-            dont_shuffle = []
-            to_shuffle = []
-            for p in generated_params:
-                if p["learner"]["model_type"] in [
-                    "Baseline",
-                    "Linear",
-                    "Decision Tree",
-                ]:
-                    dont_shuffle += [p]
-                else:
-                    to_shuffle += [p]
-
-            np.random.shuffle(to_shuffle)
-            generated_params = dont_shuffle + to_shuffle
+            # 2. Default parameters
+            self._fit_level = "default_algorithms"
+            start = time.time()
+            self._time_start[self._fit_level] = start
+            for params in tuner.default_params(len(self._models)):
+                self.train_model(params)
+            self._time_spend["default_algorithms"] = np.round(time.time() - start, 2)
 
+            # 3. The not-so-random step
+            self._fit_level = "not_so_random"
+            start = time.time()
+            self._time_start[self._fit_level] = start
+            generated_params = tuner.get_not_so_random_params(len(self._models))
             for params in generated_params:
                 self.train_model(params)
-            # hill climbing
+            self._time_spend["not_so_random"] = np.round(time.time() - start, 2)
+
+            # 4. The hill-climbing step
+            self._fit_level = "hill_climbing"
+            start = time.time()
+            self._time_start[self._fit_level] = start
             for params in tuner.get_hill_climbing_params(self._models):
                 self.train_model(params)
+            self._time_spend["hill_climbing"] = np.round(time.time() - start, 2)
+            
 
+            # 5. Ensemble unstacked models
+            self._fit_level = "ensemble_unstacked"
+            start = time.time()
+            self._time_start[self._fit_level] = start
             self.ensemble_step()
+            self._time_spend["ensemble_unstacked"] = np.round(time.time() - start, 2)
+
+            
+            if self._stack:
+                # 6. Stack best models
+                self._fit_level = "stack"
+                start = time.time()
+                self._time_start[self._fit_level] = start
+                self.stacked_ensemble_step()
+                self._time_spend["stack"] = np.round(time.time() - start, 2)
+
+                # 7. Ensemble all models (original and stacked)
+                any_stacked = False
+                for m in self._models:
+                    if m._is_stacked:
+                        any_stacked = True
+                        break
+                if any_stacked:
+                    self._fit_level = "ensemble_all"
+                    start = time.time()
+                    self.ensemble_step(is_stacked=True)
+                    self._time_spend["ensemble_all"] = np.round(time.time() - start, 2)
 
             self._fit_time = time.time() - self._start_time
 
+            logger.info(f"AutoML fit time: {self._fit_time}")
+
         except Exception as e:
             raise e
         finally:
@@ -727,6 +1012,8 @@ class AutoML:
                 "optimize_metric": self._optimize_metric,
                 "saved": self._model_paths,
             }
+            if self._stacked_models is not None:
+                params["stacked"] = [m.get_name() for m in self._stacked_models]
             fout.write(json.dumps(params, indent=4))
 
         ldb = self.get_leaderboard()
@@ -735,9 +1022,8 @@ class AutoML:
         # save report
         ldb["Link"] = [f"[Results link]({m}/README.md)" for m in ldb["name"].values]
         ldb.insert(loc=0, column="Best model", value="")
-        ldb.loc[
-            ldb.name == self._best_model.get_name(), "Best model"
-        ] = "**the best**"
+        ldb.loc[ldb.name == self._best_model.get_name(), "Best model"] = "**the best**"
+
         with open(os.path.join(self._results_path, "README.md"), "w") as fout:
             fout.write(f"# AutoML Leaderboard\n\n")
             fout.write(tabulate(ldb.values, ldb.columns, tablefmt="pipe"))
@@ -763,7 +1049,17 @@ class AutoML:
                 )
         X = X[self._data_info["columns"]]
 
-        predictions = self._best_model.predict(X)
+        # is stacked model
+        if self._best_model._is_stacked:
+            self.stack_models()
+            X_stacked = self.get_stacked_data(X, mode="predict")
+            if self._best_model.get_type() == "Ensemble":
+                # Ensemble is using both original and stacked data
+                predictions = self._best_model.predict(X, X_stacked)
+            else:
+                predictions = self._best_model.predict(X_stacked)
+        else:
+            predictions = self._best_model.predict(X)
 
         if self._ml_task == BINARY_CLASSIFICATION:
             # need to predict the label based on predictions and threshold
